@@ -43,6 +43,7 @@ enum {
 static void soup_body_output_stream_pollable_init (GPollableOutputStreamInterface *pollable_interface, gpointer interface_data);
 
 G_DEFINE_TYPE_WITH_CODE (SoupBodyOutputStream, soup_body_output_stream, G_TYPE_FILTER_OUTPUT_STREAM,
+                         G_ADD_PRIVATE (SoupBodyOutputStream)
 			 G_IMPLEMENT_INTERFACE (G_TYPE_POLLABLE_OUTPUT_STREAM,
 						soup_body_output_stream_pollable_init))
 
@@ -50,9 +51,7 @@ G_DEFINE_TYPE_WITH_CODE (SoupBodyOutputStream, soup_body_output_stream, G_TYPE_F
 static void
 soup_body_output_stream_init (SoupBodyOutputStream *stream)
 {
-	stream->priv = G_TYPE_INSTANCE_GET_PRIVATE (stream,
-						    SOUP_TYPE_BODY_OUTPUT_STREAM,
-						    SoupBodyOutputStreamPrivate);
+	stream->priv = soup_body_output_stream_get_instance_private (stream);
 }
 
 static void
@@ -163,7 +162,6 @@ again:
 	case SOUP_BODY_OUTPUT_STREAM_STATE_CHUNK_SIZE:
 		g_snprintf (buf, sizeof (bostream->priv->buf),
 			    "%lx\r\n", (gulong)count);
-		len = strlen (buf);
 
 		if (count > 0)
 			bostream->priv->chunked_state = SOUP_BODY_OUTPUT_STREAM_STATE_CHUNK;
@@ -183,13 +181,11 @@ again:
 
 	case SOUP_BODY_OUTPUT_STREAM_STATE_CHUNK_END:
 		strncpy (buf, "\r\n", sizeof (bostream->priv->buf));
-		len = 2;
 		bostream->priv->chunked_state = SOUP_BODY_OUTPUT_STREAM_STATE_DONE;
 		break;
 
 	case SOUP_BODY_OUTPUT_STREAM_STATE_TRAILERS:
 		strncpy (buf, "\r\n", sizeof (bostream->priv->buf));
-		len = 2;
 		bostream->priv->chunked_state = SOUP_BODY_OUTPUT_STREAM_STATE_DONE;
 		break;
 
@@ -231,7 +227,8 @@ soup_body_output_stream_close_fn (GOutputStream  *stream,
 {
 	SoupBodyOutputStream *bostream = SOUP_BODY_OUTPUT_STREAM (stream);
 
-	if (bostream->priv->encoding == SOUP_ENCODING_CHUNKED) {
+	if (bostream->priv->encoding == SOUP_ENCODING_CHUNKED &&
+	    bostream->priv->chunked_state == SOUP_BODY_OUTPUT_STREAM_STATE_CHUNK_SIZE) {
 		if (soup_body_output_stream_write_chunked (bostream, NULL, 0, TRUE, cancellable, error) == -1)
 			return FALSE;
 	}
@@ -296,8 +293,6 @@ soup_body_output_stream_class_init (SoupBodyOutputStreamClass *stream_class)
 	GObjectClass *object_class = G_OBJECT_CLASS (stream_class);
 	GOutputStreamClass *output_stream_class = G_OUTPUT_STREAM_CLASS (stream_class);
 
-	g_type_class_add_private (stream_class, sizeof (SoupBodyOutputStreamPrivate));
-
 	object_class->constructed = soup_body_output_stream_constructed;
 	object_class->set_property = soup_body_output_stream_set_property;
 	object_class->get_property = soup_body_output_stream_get_property;
@@ -312,14 +307,16 @@ soup_body_output_stream_class_init (SoupBodyOutputStreamClass *stream_class)
 				   "Message body encoding",
 				   SOUP_TYPE_ENCODING,
 				   SOUP_ENCODING_NONE,
-				   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+				   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
+				   G_PARAM_STATIC_STRINGS));
 	g_object_class_install_property (
 		object_class, PROP_CONTENT_LENGTH,
 		g_param_spec_uint64 ("content-length",
 				     "Content-Length",
 				     "Message body Content-Length",
 				     0, G_MAXUINT64, 0,
-				     G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+				     G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY |
+				     G_PARAM_STATIC_STRINGS));
 }
 
 static void
