@@ -3,8 +3,8 @@
  * Copyright (C) 2008 Red Hat, Inc.
  */
 
-#ifndef SOUP_LOGGER_H
-#define SOUP_LOGGER_H 1
+#ifndef __SOUP_LOGGER_H__
+#define __SOUP_LOGGER_H__ 1
 
 #include <libsoup/soup-types.h>
 
@@ -49,27 +49,39 @@ typedef void               (*SoupLoggerPrinter) (SoupLogger         *logger,
 						 const char         *data,
 						 gpointer            user_data);
 
+SOUP_AVAILABLE_IN_2_4
 GType       soup_logger_get_type    (void);
 
+#define SOUP_LOGGER_LEVEL         "level"
+#define SOUP_LOGGER_MAX_BODY_SIZE "max-body-size"
+
+SOUP_AVAILABLE_IN_2_4
 SoupLogger *soup_logger_new         (SoupLoggerLogLevel  level,
 				     int                 max_body_size);
 
+#ifndef SOUP_DISABLE_DEPRECATED
+SOUP_AVAILABLE_IN_2_4
 SOUP_DEPRECATED_IN_2_24_FOR(soup_session_add_feature)
 void        soup_logger_attach      (SoupLogger         *logger,
 				     SoupSession        *session);
+SOUP_AVAILABLE_IN_2_4
 SOUP_DEPRECATED_IN_2_24_FOR(soup_session_remove_feature)
 void        soup_logger_detach      (SoupLogger         *logger,
 				     SoupSession        *session);
+#endif
 
+SOUP_AVAILABLE_IN_2_4
 void        soup_logger_set_request_filter  (SoupLogger        *logger,
 					     SoupLoggerFilter   request_filter,
 					     gpointer           filter_data,
 					     GDestroyNotify     destroy);
+SOUP_AVAILABLE_IN_2_4
 void        soup_logger_set_response_filter (SoupLogger        *logger,
 					     SoupLoggerFilter   response_filter,
 					     gpointer           filter_data,
 					     GDestroyNotify     destroy);
 
+SOUP_AVAILABLE_IN_2_4
 void        soup_logger_set_printer         (SoupLogger        *logger,
 					     SoupLoggerPrinter  printer,
 					     gpointer           printer_data,
@@ -77,4 +89,4 @@ void        soup_logger_set_printer         (SoupLogger        *logger,
 
 G_END_DECLS
 
-#endif /* SOUP_LOGGER_H */
+#endif /* __SOUP_LOGGER_H__ */
