@@ -3,8 +3,8 @@
  * Copyright 2011-2014 Red Hat, Inc.
  */
 
-#ifndef SOUP_SOCKET_PRIVATE_H
-#define SOUP_SOCKET_PRIVATE_H 1
+#ifndef __SOUP_SOCKET_PRIVATE_H__
+#define __SOUP_SOCKET_PRIVATE_H__ 1
 
 #include "soup-socket.h"
 
@@ -38,7 +38,6 @@ gboolean   soup_socket_handshake_finish        (SoupSocket           *sock,
 						GAsyncResult         *result,
 						GError              **error);
 
-gboolean   soup_socket_is_readable             (SoupSocket           *sock);
 GSocket   *soup_socket_get_gsocket             (SoupSocket           *sock);
 GSocket   *soup_socket_steal_gsocket           (SoupSocket           *sock);
 GIOStream *soup_socket_get_connection          (SoupSocket           *sock);
@@ -46,6 +45,10 @@ GIOStream *soup_socket_get_iostream            (SoupSocket           *sock);
 
 SoupURI   *soup_socket_get_http_proxy_uri      (SoupSocket           *sock);
 
+gboolean   soup_socket_listen_full             (SoupSocket           *sock,
+                                                GError              **error);
+
+
 
 typedef struct {
 	GMainContext *async_context;
@@ -84,4 +87,4 @@ void                  soup_socket_properties_unref (SoupSocketProperties *props)
 void soup_socket_properties_push_async_context (SoupSocketProperties *props);
 void soup_socket_properties_pop_async_context  (SoupSocketProperties *props);
 
-#endif /* SOUP_SOCKET_PRIVATE_H */
+#endif /* __SOUP_SOCKET_PRIVATE_H__ */
