@@ -54,18 +54,26 @@ do_session_property_tests (void)
 	GTlsDatabase *tlsdb;
 	char *ca_file;
 	SoupSession *session;
+	GParamSpec *pspec;
 
 	g_test_bug ("700518");
 
+	G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
 	session = soup_session_async_new ();
+	G_GNUC_END_IGNORE_DEPRECATIONS;
+
+	/* Temporarily undeprecate SOUP_SESSION_SSL_CA_FILE to avoid warnings. */
+	pspec = g_object_class_find_property (g_type_class_peek (SOUP_TYPE_SESSION),
+					      SOUP_SESSION_SSL_CA_FILE);
+	pspec->flags &= ~G_PARAM_DEPRECATED;
 
 	g_object_get (G_OBJECT (session),
 		      "ssl-use-system-ca-file", &use_system,
 		      "tls-database", &tlsdb,
 		      "ssl-ca-file", &ca_file,
 		      NULL);
-	soup_test_assert (!use_system, "ssl-use-system-ca-file defaults to TRUE");
-	soup_test_assert (tlsdb == NULL, "tls-database set by default");
+	soup_test_assert (use_system, "ssl-use-system-ca-file defaults to FALSE");
+	soup_test_assert (tlsdb != NULL, "tls-database not set by default");
 	soup_test_assert (ca_file == NULL, "ca-file set by default");
 
 	g_object_set (G_OBJECT (session),
@@ -102,6 +110,9 @@ do_session_property_tests (void)
 	soup_test_assert (ca_file == NULL, "setting tls-database NULL set ssl-ca-file");
 
 	soup_test_session_abort_unref (session);
+
+	/* Re-deprecate SOUP_SESSION_SSL_CA_FILE */
+	pspec->flags |= G_PARAM_DEPRECATED;
 }
 
 static void
